"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.StepFunctionSimulatorServer = void 0;
const http_terminator_1 = require("http-terminator");
const express_1 = __importDefault(require("express"));
const StateMachineExecutor_1 = require("./StateMachineExecutor");
const Logger_1 = require("./utils/Logger");
class StepFunctionSimulatorServer {
    constructor(options) {
        this.options = options;
        this.logger = Logger_1.Logger.getInstance();
        this.express = express_1.default();
        this.setupMiddlewares();
    }
    async initServer() {
        let httpServer;
        try {
            httpServer = this.express.listen(this.options.port, () => {
                this.logger.success(`Server ready on port ${this.options.port} 🚀`);
            });
        }
        catch (err) {
            this.logger.error(`Unexpected error while starting serverless-offline server on port ${this.options.port}: ${err.stack}`);
            process.exit(1);
        }
        this.httpTerminator = http_terminator_1.createHttpTerminator({ server: httpServer });
    }
    async shutdown() {
        var _a;
        this.logger.warning('Killing Step Functions API Simulator 🔪');
        await ((_a = this.httpTerminator) === null || _a === void 0 ? void 0 : _a.terminate());
    }
    setupMiddlewares() {
        this.express.use(express_1.default.json({
            type(req) {
                const contentType = req.headers['content-type'] || '';
                return ['application/x-amz-json-1.0'].includes(contentType);
            },
        }));
        this.express.use(this.resolveStateMachine.bind(this));
    }
    async resolveStateMachine(req, res) {
        this.logger.log(`Got request for ${req.method} ${req.url}`);
        const executionInput = req.body;
        const stateMachineName = executionInput.stateMachineArn.split(':').slice(-1)[0];
        const stateMachineToExecute = this.options.stateMachines[stateMachineName];
        if (!stateMachineToExecute) {
            return res.status(500);
        }
        const startAtState = stateMachineToExecute.definition.States[stateMachineToExecute.definition.StartAt];
        const sme = new StateMachineExecutor_1.StateMachineExecutor(stateMachineToExecute);
        await new Promise((resolve) => {
            const output = {
                startDate: sme.startDate,
                executionArn: sme.executionArn,
            };
            resolve(res.status(200).json(output));
        });
        await sme.execute(startAtState, executionInput.input);
    }
}
exports.StepFunctionSimulatorServer = StepFunctionSimulatorServer;
